@@ -24,6 +24,38 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: sipep.cxx,v $
+ * Revision 2.97.2.39  2007/04/21 10:43:17  dsandras
+ * Fixed more interoperability problems due to bugs in Cisco Call Manager.
+ * Do not clear calls if the video transport can not be established.
+ * Only reinitialize the registrar transport if required (STUN is being used
+ * and the public IP address has changed).
+ *
+ * Revision 2.97.2.38  2007/04/17 21:50:19  dsandras
+ * Fixed Via field in previous commit.
+ * Make sure the correct port is being used.
+ * Improved FindSIPInfoByDomain.
+ *
+ * Revision 2.97.2.37  2007/04/15 09:54:46  dsandras
+ * Some systems like CISCO Call Manager do like having a Contact field in INVITE
+ * PDUs which is different to the one being used in the original REGISTER request.
+ * Added code to use the same Contact field in both cases if we can determine that
+ * we are registered to that specific account and if there is a transport running.
+ * Fixed problem where the SIP connection was not released with a BYE PDU when
+ * the ACK is received while we are already in EstablishedPhase.
+ *
+ * Revision 2.97.2.36  2007/03/27 21:52:45  dsandras
+ * Added more PTRACE statements
+ *
+ * Revision 2.97.2.35  2007/03/27 20:23:22  dsandras
+ * Added Garbage collector. Make sure the transport is closed before
+ * deleting it. Better handling of SIPInfo objects.
+ *
+ * Revision 2.97.2.34  2007/03/17 14:59:41  dsandras
+ * Keep the transport open for a longer time when doing text conversations.
+ *
+ * Revision 2.97.2.33  2007/03/13 21:23:24  dsandras
+ * Backport from HEAD.
+ *
  * Revision 2.97.2.32  2007/03/10 16:48:21  dsandras
  * Improved locking.
  *
@@ -574,7 +606,11 @@ SIPInfo::~SIPInfo()
 {
   PWaitAndSignal m(transportMutex);
 
+  PTRACE(4,"SIPInfo\tWill delete SIPInfo " << registrationAddress);
   if (registrarTransport) { 
+
+    PTRACE(4,"SIPInfo\tWill delete transport " << *registrarTransport << " (deleting SIPInfo)");
+    registrarTransport->CloseWait ();
     delete registrarTransport;
     registrarTransport = NULL;
   }
@@ -588,13 +624,28 @@ BOOL SIPInfo::CreateTransport (OpalTransportAddress & transportAddress)
   PWaitAndSignal m(transportMutex);
 
   registrarAddress = transportAddress;
-  
-  // Only delete if we are refreshing
-  if (registrarTransport != NULL && HasExpired()) {
-    delete registrarTransport;
-    registrarTransport = NULL;
-  }
 
+  if (registrarTransport != NULL) {
+    if (!registrarTransport->IsReliable()) {
+      PSTUNClient *stunClient = ep.GetManager().GetSTUN (transportAddress.GetHostName());
+      if (stunClient) {
+        PIPSocket::Address externalAddress;
+        PIPSocket::Address currentAddress;
+        WORD port = 5060;
+        if (registrarTransport->GetLocalAddress().GetIpAndPort (currentAddress, port)) {
+          if (stunClient->GetExternalAddress (externalAddress, 10)) {
+            if (externalAddress != currentAddress) {
+              PTRACE(4,"SIPInfo\tWill delete transport " << *registrarTransport << " (external IP address changed)");
+              registrarTransport->CloseWait ();
+              delete registrarTransport;
+              registrarTransport = NULL;
+            }
+          }
+        }
+      }
+    }
+  }
+  
   if (registrarTransport == NULL) {
     registrarTransport = ep.CreateTransport(registrarAddress);
   }
@@ -650,7 +701,7 @@ SIPTransaction * SIPRegisterInfo::CreateTransaction(OpalTransport &t, BOOL unreg
 			  registrationAddress, 
 			  registrationID, 
 			  unregister ? 0 : expire,
-        retryTimeoutMin, retryTimeoutMax);
+                          retryTimeoutMin, retryTimeoutMax);
 }
 
 void SIPRegisterInfo::OnSuccess ()
@@ -722,6 +773,7 @@ SIPTransaction * SIPMessageInfo::CreateTransaction(OpalTransport &t, BOOL /*unre
 
 void SIPMessageInfo::OnSuccess()
 { 
+  SetRegistered(TRUE);
 }
 
 void SIPMessageInfo::OnFailed(SIP_PDU::StatusCodes reason)
@@ -753,7 +805,10 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr)
   activeSIPInfo.AllowDeleteObjects();
 
   registrationTimer.SetNotifier(PCREATE_NOTIFIER(RegistrationRefresh));
-  registrationTimer.RunContinuous (PTimeInterval(0, 2));
+  registrationTimer.RunContinuous (PTimeInterval(0, 30));
+
+  garbageTimer.SetNotifier(PCREATE_NOTIFIER(GarbageCollect));
+  garbageTimer.RunContinuous (PTimeInterval(0, 2));
   
   natBindingTimer.SetNotifier(PCREATE_NOTIFIER(NATBindingRefresh));
   natBindingTimer.RunContinuous (natBindingTimeout);
@@ -775,7 +830,7 @@ SIPEndPoint::~SIPEndPoint()
       if (info->GetMethod() == SIP_PDU::Method_REGISTER) {
 
         if (info->IsRegistered() && info->GetExpire() > 0) {
-          Unregister(url.GetHostName(), url.GetUserName());
+          Unregister(url.AsString(), url.AsString());
         }
         else if (!info->IsRegistered()){ 
           info->SetExpire(-1);
@@ -829,7 +884,7 @@ void SIPEndPoint::TransportThreadMain(PThread &, INT param)
 
   do {
     HandlePDU(*transport);
-  } while (transport->IsOpen());
+  } while (transport->IsOpen() && !transport->bad() && !transport->eof());
   
   PTRACE(2, "SIP\tRead thread finished.");
 }
@@ -1379,6 +1434,12 @@ void SIPEndPoint::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
  
   if (info == NULL) 
     return;
+
+  if (transaction.GetMethod() == SIP_PDU::Method_MESSAGE) {
+    info->OnSuccess();
+    return;
+  }
+
   
   // reset the number of unsuccesful authentication attempts
   info->SetAuthenticationAttempts(0);
@@ -1400,14 +1461,15 @@ void SIPEndPoint::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
 
     if (info->GetAuthentication().GetAuthRealm().IsEmpty())
       info->SetAuthRealm(transaction.GetURI().GetHostName());
+
+    info->OnSuccess();
   }
   else { 
-    info->SetExpire(-1);
     info->SetRegistered(FALSE);
-  }
+    info->OnSuccess();
 
-  // Callback
-  info->OnSuccess();
+    info->SetExpire(-1);
+  }
 }
 
 
@@ -1585,6 +1647,19 @@ BOOL SIPEndPoint::IsSubscribed(const PString & host, const PString & user)
 }
 
 
+void SIPEndPoint::GarbageCollect(PTimer &, INT)
+{
+  for (PINDEX i = 0 ; i < activeSIPInfo.GetSize () ; i++) {
+
+    PSafePtr<SIPInfo> info = activeSIPInfo.GetAt (i, PSafeReadWrite);
+
+    if (info->GetExpire() == -1) {
+      activeSIPInfo.Remove(info); // Was invalid the last time, delete it
+    }
+  }
+}
+
+
 void SIPEndPoint::RegistrationRefresh(PTimer &, INT)
 {
   SIPTransaction *request = NULL;
@@ -1593,47 +1668,41 @@ void SIPEndPoint::RegistrationRefresh(PTimer &, INT)
   // Timer has elapsed
   for (PINDEX i = 0 ; i < activeSIPInfo.GetSize () ; i++) {
 
-    PSafePtr<SIPInfo> info = activeSIPInfo.GetAt (i, PSafeReadWrite);
+    PSafePtr<SIPInfo> info = activeSIPInfo.GetAt (i, PSafeReadOnly);
 
-    if (info->GetExpire() == -1) {
-      activeSIPInfo.Remove(info); // Was invalid the last time, delete it
+    // Need to refresh
+    if (info->GetExpire() > 0 
+        && info->IsRegistered()
+        && info->GetTransport() != NULL 
+        && info->GetMethod() != SIP_PDU::Method_MESSAGE
+        && info->HasExpired()) {
+      PTRACE(2, "SIP\tStarting REGISTER/SUBSCRIBE for binding refresh");
+      infoTransport = info->GetTransport(); // Get current transport
+      OpalTransportAddress registrarAddress = infoTransport->GetRemoteAddress();
+      // Will update the transport if required. For example, if STUN
+      // is used, and the external IP changed. Otherwise, OPAL would
+      // keep registering the old IP.
+      if (info->CreateTransport(registrarAddress)) { 
+        infoTransport = info->GetTransport();
+        info->RemoveTransactions();
+        info->SetExpire(info->GetExpire()*10/9);
+        request = info->CreateTransaction(*infoTransport, FALSE); 
+
+        if (request->Start()) 
+          info->AppendTransaction(request);
+        else {
+          delete request;
+          PTRACE(1, "SIP\tCould not start REGISTER/SUBSCRIBE for binding refresh");
+          info->SetExpire(-1); // Mark as Invalid
+        }
+      }
+      else {
+        PTRACE(1, "SIP\tCould not start REGISTER/SUBSCRIBE for binding refresh: Transport creation failed");
+        info->SetExpire(-1); // Mark as Invalid
+      }
     }
-    else {
-
-      // Need to refresh
-      if (info->GetExpire() > 0 
-	  && info->IsRegistered()
-	  && info->GetTransport() != NULL 
-	  && info->GetMethod() != SIP_PDU::Method_MESSAGE
-	  && info->HasExpired()) {
-	PTRACE(2, "SIP\tStarting REGISTER/SUBSCRIBE for binding refresh");
-	infoTransport = info->GetTransport(); // Get current transport
-	OpalTransportAddress registrarAddress = infoTransport->GetRemoteAddress();
-	// Will update the transport if required. For example, if STUN
-	// is used, and the external IP changed. Otherwise, OPAL would
-	// keep registering the old IP.
-	if (info->CreateTransport(registrarAddress)) { 
-	  infoTransport = info->GetTransport();
-	  info->RemoveTransactions();
-	  info->SetExpire(info->GetExpire()*10/9);
-	  request = info->CreateTransaction(*infoTransport, FALSE); 
-
-	  if (request->Start()) 
-	    info->AppendTransaction(request);
-	  else {
-	    delete request;
-	    PTRACE(1, "SIP\tCould not start REGISTER/SUBSCRIBE for binding refresh");
-	    info->SetExpire(-1); // Mark as Invalid
-	  }
-	}
-	else {
-	  PTRACE(1, "SIP\tCould not start REGISTER/SUBSCRIBE for binding refresh: Transport creation failed");
-	  info->SetExpire(-1); // Mark as Invalid
-	}
-      }
-      else if (info->HasExpired()) {
-	info->SetExpire(-1); // Mark as Invalid
-      }
+    else if (info->HasExpired()) {
+      info->SetExpire(-1); // Mark as Invalid
     }
   }
 
@@ -1986,6 +2055,17 @@ SIPURL SIPEndPoint::GetDefaultRegisteredPartyName()
 }
 
 
+SIPURL SIPEndPoint::GetContactURL(const OpalTransport &transport, const PString & userName, const PString & host)
+{
+  PSafePtr<SIPInfo> info = activeSIPInfo.FindSIPInfoByDomain(host, SIP_PDU::Method_REGISTER, PSafeReadOnly);
+  
+  if (info == NULL || info->GetTransport() == NULL) 
+    return GetLocalURL(transport, userName);
+  else
+    return GetLocalURL(*info->GetTransport(), userName);
+}
+
+
 SIPURL SIPEndPoint::GetLocalURL(const OpalTransport &transport, const PString & userName)
 {
   PIPSocket::Address ip(PIPSocket::GetDefaultIpAny());
@@ -2003,10 +2083,7 @@ SIPURL SIPEndPoint::GetLocalURL(const OpalTransport &transport, const PString &
     PIPSocket::Address remoteIP;
     if (transport.GetRemoteAddress().GetIpAddress(remoteIP)) {
       GetManager().TranslateIPAddress(localIP, remoteIP); 	 
-      PIPSocket::Address _localIP(localIP);
-      PSTUNClient * stun = manager.GetSTUN(remoteIP);
-      if (stun != NULL || localIP != _localIP)
-	contactPort = localPort;
+      contactPort = localPort;
       contactAddress = OpalTransportAddress(localIP, contactPort, "udp");
     }
   }
@@ -2020,7 +2097,7 @@ SIPURL SIPEndPoint::GetLocalURL(const OpalTransport &transport, const PString &
 BOOL SIPEndPoint::SendMessage (const SIPURL & url, 
 			       const PString & body)
 {
-  return TransmitSIPInfo(SIP_PDU::Method_MESSAGE, url.AsQuotedString(), url.GetUserName(), "", "", "", body);
+  return TransmitSIPInfo(SIP_PDU::Method_MESSAGE, url.AsQuotedString(), url.GetUserName(), "", "", "", body, 500);
 }
 
 
